@@ -12,7 +12,8 @@ from textwrap import fill
 
 import numpy as np
 
-from .utils import calculate_WAIC, congrid, is_integer, pretty_num
+from .utils import (calculate_WAIC, congrid, is_integer, open_atomic,
+                    pretty_num, rebin)
 
 if sys.version_info[:2] < (3, 3):
     old_print = print  # noqa
@@ -64,6 +65,7 @@ class Printer(object):
         self._pool = pool
         self._fitter = fitter
         self._language = language
+        self._was_inline = False
 
         self.set_strings()
 
@@ -97,7 +99,7 @@ class Printer(object):
             self._strings = {}
             for key in strings:
                 self._strings[key] = self.translate(strings[key])
-            with open(lsf, 'w') as f:
+            with open_atomic(lsf, 'w') as f:
                 json.dump(self._strings, f)
 
     def set_language(self, language):
@@ -153,9 +155,13 @@ class Printer(object):
         if inline and self._fitter is not None:
             inline = not self._fitter._test
         if inline:
-            for line in tspl:
-                sys.stdout.write("\033[F")
-                sys.stdout.write("\033[K")
+            if self._was_inline:
+                for line in tspl:
+                    sys.stdout.write("\033[F")
+                    sys.stdout.write("\033[K")
+            self._was_inline = True
+        else:
+            self._was_inline = False
         for ri, line in enumerate(tspl):
             rline = line
             if colorify:
@@ -352,8 +358,12 @@ class Printer(object):
         outarr.extend(messages)
 
         kmat_extra = 0
-        if kmat is not None:
-            kmat_scaled = congrid(kmat, (14, 7))
+        if kmat is not None and kmat.shape[0] > 1:
+            try:
+                kmat_scaled = congrid(kmat, (14, 7), minusone=True,
+                                      bounds_error=True)
+            except Exception:
+                kmat_scaled = rebin(kmat, (14, 7))
             kmat_scaled = np.log(kmat_scaled)
             kmat_scaled /= np.max(kmat_scaled)
             kmat_pers = [np.percentile(kmat_scaled, x) for x in (20, 50, 80)]
@@ -382,7 +392,7 @@ class Printer(object):
 
         lines = lines + '\n' + line
 
-        if kmat is not None:
+        if kmat is not None and kmat.shape[0] > 1:
             lines = self._lines(lines)
             loff = int(np.floor((len(kmat_scaled[0]) - len(lines)) / 2.0)) + 2
             for li, line in enumerate(doodle):
@@ -394,6 +404,8 @@ class Printer(object):
             lines = '\n'.join(doodle)
 
         self.prt(lines, colorify=True, inline=not make_space)
+        if make_space:
+            self._was_inline = True
 
     def get_timestring(self, t):
         """Return estimated time remaining.
